/**
 * 仿射变换矩阵支持
 *
 * Created by slayer on 18-12-26.
 */

 class Affine {
  /**
   * 初始化一个标准矩阵
   *
   *
   */
  constructor() {
    this.mA = 1;
    this.mB = 0;
    this.mC = 0;
    this.mD = 1;
    this.mE = 0;
    this.mF = 0;
  }

  setElement(_mA, _mB, _mC, _mD, _mE, _mF) {
    this.mA = _mA;
    this.mB = _mB;
    this.mC = _mC;
    this.mD = _mD;
    this.mE = _mE;
    this.mF = _mF;
  }

  toMatrixString() {
    return (
      "matrix(" +
      this.mA +
      "," +
      this.mB +
      "," +
      this.mC +
      "," +
      this.mD +
      "," +
      this.mE +
      "," +
      this.mF +
      ")"
    );
  }

  /**
   * 从 SXAffine 生成一个矩阵
   *
   */
  static fromSXAffine(originAffine) {
    var affine = new Affine();

    affine.mA = originAffine[0];
    affine.mB = originAffine[1];
    affine.mC = originAffine[2];
    affine.mD = originAffine[3];
    affine.mE = originAffine[4];
    affine.mF = originAffine[5];

    return affine;
  }

  /**
   * 转换成 SXAffine
   *
   */
  toSXAffine() {
    return [this.mA, this.mB, this.mC, this.mD, this.mE, this.mF];
  }

  /**
   * 从另一个 Affine 矩阵复制
   *
   */
  cloneFrom(originAffine) {
    this.mA = originAffine.mA;
    this.mB = originAffine.mB;
    this.mC = originAffine.mC;
    this.mD = originAffine.mD;
    this.mE = originAffine.mE;
    this.mF = originAffine.mF;
  }

  /**
   * 设置锚点,位置,缩放,旋转
   *
   * @param anchor { x : 0, y : 0}
   * @param position { x : 0, y : 0}
   * @param scale { x : 0, y : 0}
   * @param rotation 角度
   */
  set(anchor, position, scale, rotation) {
    let _rotation = Affine.degreeToRadians(rotation);
    let cs = Math.cos(_rotation);
    let sn = Math.sin(_rotation);

    this.mA = cs * scale.x;
    this.mB = sn * scale.x;
    this.mC = -sn * scale.y;
    this.mD = cs * scale.y;

    this.mE = position.x;
    this.mF = position.y;

    this.translate(-anchor.x, -anchor.y);
  }

  /**
   * 移动位置
   *
   */
  translate(tx, ty) {
    let t = new Affine();
    t.setTranslate(tx, ty);
    this.premultiply(t);
  }

  /**
   * 旋转
   *
   */
  rotate(degree) {
    var t = new Affine();
    t.setRotate(degree);
    this.premultiply(t);
  }

  /**
   * 缩放
   *
   */
  scale(sx, sy) {
    var t = new Affine();
    t.setScale(sx, sy);
    this.premultiply(t);
  }

  /**
   * 重置为标准矩阵,并设置位置
   *
   */
  setTranslate(tx, ty) {
    this.mA = this.mD = 1;
    this.mB = this.mC = 0;
    this.mE = tx;
    this.mF = ty;
  }

  /**
   * 重置为标准矩阵,并设置旋转
   *
   */
  setRotate(degree) {
    var r = this.degreeToRadians(degree);
    var cs = Math.cos(degree);
    var sn = Math.sin(degree);
    this.mA = cs;
    this.mB = sn;
    this.mC = -sn;
    this.mD = cs;
    this.mE = 0;
    this.mF = 0;
  }

  /**
   * 重置为标准矩阵,并设置缩放
   *
   */
  setScale(sx, sy) {
    this.mA = sx;
    this.mD = sy;
    this.mB = this.mC = this.mE = this.mF = 0;
  }

  /**
   * 角度转弧度
   *
   */
  static degreeToRadians(degree) {
    return degree * 0.017453292519943;
  }

  static radiansToDegree(radians) {
    return radians * 57.29577951308232;
  }

  /**
   * 预乘
   *
   */
  premultiply(t) {
    t.multiply(this);
    this.mA = t.mA;
    this.mB = t.mB;
    this.mC = t.mC;
    this.mD = t.mD;
    this.mE = t.mE;
    this.mF = t.mF;
  }

  /**
   * 后乘
   *
   */
  multiply(other) {
    let t0 = this.mA * other.mA + this.mB * other.mC;
    let t2 = this.mC * other.mA + this.mD * other.mC;
    let t4 = this.mE * other.mA + this.mF * other.mC + other.mE;

    this.mB = this.mA * other.mB + this.mB * other.mD;
    this.mD = this.mC * other.mB + this.mD * other.mD;
    this.mF = this.mE * other.mB + this.mF * other.mD + other.mF;

    this.mA = t0;
    this.mC = t2;
    this.mE = t4;
  }

  /**
   * 获取逆矩阵
   *
   */
  getInverse() {
    let inv_det;
    let det = this.mA * this.mD - this.mC * this.mB;
    if (det > -0.000001 && det < 0.000001) {
      return false;
    }
    inv_det = 1.0 / det;

    let out = new Affine();
    out.mA = this.mD * inv_det;
    out.mC = -this.mC * inv_det;
    out.mE = (this.mC * this.mF - this.mD * this.mE) * inv_det;
    out.mB = -this.mB * inv_det;
    out.mD = this.mA * inv_det;
    out.mF = (this.mB * this.mE - this.mA * this.mF) * inv_det;

    return out;
  }

  transform(position) {
    let pos = {};
    pos.x = this.mA * position.x + this.mC * position.y + this.mE;
    pos.y = this.mB * position.x + this.mD * position.y + this.mF;

    return pos;
  }

  getPositionFromAnchor(ax, ay) {
    let result = new Affine();
    result.cloneFrom(this);
    result.translate(ax, ay);
    return { x: result.mE, y: result.mF };
  }

  setPositionFromAnchor(ax, ay) {
    this.translate(ax, ay);
    return { x: this.mE, y: this.mF };
  }

  static rotateAround(p, angle, center) {
    if (angle == 0) {
      return p;
    }

    let pp = {
      x: p.x - center.x,
      y: p.y - center.y,
    };
    let r = Affine.degreeToRadians(angle);
    let ca = Math.cos(r);
    let sa = Math.sin(r);

    return {
      x: center.x + pp.x * ca - pp.y * sa,
      y: center.y + pp.x * sa + pp.y * ca,
    };
  }

  static angleBetween(p1, p2) {
    let cross = p1.x * p2.y - p1.y * p2.x;
    let dot = p1.x * p2.x + p1.y * p2.y;
    return Affine.radiansToDegree(Math.atan2(cross, dot));
  }

  decomposeWithAnchor(ax, ay) {
    let a = this.mA;
    let b = this.mB;
    let c = this.mC;
    let d = this.mD;

    let result = {
      position: this.getPositionFromAnchor(ax, ay),
      rotate: 0,
      scale: { x: 1, y: 1 },
      anchor: { x: ax, y: ay },
    };

    let delta = a * d - b * c;

    if (a != 0 || b != 0) {
      var r = Math.sqrt(a * a + b * b);
      result.rotate = b > 0 ? Math.acos(a / r) : -Math.acos(a / r);
      result.scale = { x: r, y: delta / r };
    } else if (c != 0 || d != 0) {
      var s = Math.sqrt(c * c + d * d);
      result.rotate =
        Math.PI / 2 - (d > 0 ? Math.acos(-c / s) : -Math.acos(c / s));
      result.scale = { x: delta / s, y: s };
    } else {
      // a = b = c = d = 0
    }

    result.rotate *= 180 / Math.PI;
    return result;
  }

  decompose() {
    return this.decomposeWithAnchor(0, 0);
  }
}

export default Affine;
