import Affine from "./Affine";

/**
 * MatrixUtil, 矩阵辅助方法类
 *
 * @author slayer, at 2021-08-26
 */

export default class MatrixUtil {
  /**
   * [val0, val1] => {x : val0, y : val1}
   *
   */
  static toVec2(arr) {
    return { x: arr[0], y: arr[1] };
  }

  /**
   * {x : val0, y : val1} => [val0, val1]
   *
   */
  static toArray(vec2) {
    return [vec2.x, vec2.y];
  }

  /**
   * 获取一个Affine 矩阵
   *
   * @param matrixVar, 矩阵变量对象
   */
  static getTransformMatrix(matrixVar) {
    return MatrixUtil.composeTransformMatrix(
      matrixVar.anchor,
      matrixVar.position,
      matrixVar.scale,
      matrixVar.rotate,
    );
  }

  /**
   * 生成 Affine 矩阵
   *
   * @param anchor, 锚点, 支持 [x, y] 和 { x : val, y : val}
   * @param position, 位置 , 支持 [x, y] 和 { x : val, y : val}
   * @param scale, 缩放, 支持 [x, y] 和 { x : val, y : val}
   * @param roate, 旋转角度
   */
  static composeTransformMatrix(anchor, position, scale, rotate) {
    let affine = new Affine();
    anchor = Array.isArray(anchor) ? MatrixUtil.toVec2(anchor) : anchor;
    position = Array.isArray(position) ? MatrixUtil.toVec2(position) : position;
    scale = Array.isArray(scale) ? MatrixUtil.toVec2(scale) : scale;
    affine.set(anchor, position, scale, rotate);
    return affine;
  }

  /**
   * 自动适配素材到 editSize， 锚点会被设置到中心
   *
   * @param imageSize, 原图素材尺寸
   * @param editSize, 目标适配框大小
   * */
  static autoAdapt(imageSize, editSize) {
    let _scale = editSize[1] / imageSize[1]; // 默认按高
    let widthScale = editSize[0] / imageSize[0];
    if (_scale > widthScale) {
      _scale = widthScale;
    }

    //居中计算
    let offsetX = (editSize[0] - imageSize[0]) / 2 + imageSize[0] / 2;
    let offsetY = (editSize[1] - imageSize[1]) / 2 + imageSize[1] / 2;

    let target = {};
    target.anchor = [imageSize[0] / 2, imageSize[1] / 2];
    target.position = [offsetX, offsetY];
    target.scale = [_scale, _scale];
    target.rotate = 0;

    return target;
  }

  /**
   * 获取 scale 的信息
   *
   */
  static getScaleInfo(direction, eleSize) {
    let anchor = { x: 0, y: 0 };
    let corner = { x: 0, y: 0 };
    let isVerticalScale = false;
    let isHorizontalScale = false;

    if (direction[0] == -1 && direction[1] == -1) {
      // top left
      anchor = { x: eleSize[0], y: eleSize[1] };
      corner = { x: 0, y: 0 };
    } else if (direction[0] == 1 && direction[1] == -1) {
      // top right
      anchor = { x: 0, y: eleSize[1] };
      corner = { x: eleSize[0], y: 0 };
    } else if (direction[0] == -1 && direction[1] == 1) {
      // bottom left
      anchor = { x: eleSize[0], y: 0 };
      corner = { x: 0, y: eleSize[1] };
    } else if (direction[0] == 1 && direction[1] == 1) {
      // bottm right
      anchor = { x: 0, y: 0 };
      corner = { x: eleSize[0], y: eleSize[1] };
    } else if (direction[0] == -1 && direction[1] == 0) {
      // left
      anchor = { x: eleSize[0], y: eleSize[1] / 2 };
      corner = { x: 0, y: eleSize[1] / 2 };
      isHorizontalScale = true;
    } else if (direction[0] == 1 && direction[1] == 0) {
      // right
      anchor = { x: 0, y: eleSize[1] / 2 };
      corner = { x: eleSize[0], y: eleSize[1] / 2 };
      isHorizontalScale = true;
    } else if (direction[0] == 0 && direction[1] == -1) {
      // top
      anchor = { x: eleSize[0] / 2, y: eleSize[1] };
      corner = { x: eleSize[0] / 2, y: 0 };
      isVerticalScale = true;
    } else if (direction[0] == 0 && direction[1] == 1) {
      // bottom
      anchor = { x: eleSize[0] / 2, y: 0 };
      corner = { x: eleSize[0] / 2, y: eleSize[1] };
      isVerticalScale = true;
    }

    return {
      isVerticalScale: isVerticalScale,
      isHorizontalScale: isHorizontalScale,
      corner: corner,
      anchor: anchor,
    };
  }

  static calculateScale(scaleInfo, localPos, position, rotate, referScale) {
    let { corner, anchor, isVerticalScale, isHorizontalScale } = scaleInfo;

    corner = {
      x: corner.x - anchor.x,
      y: corner.y - anchor.y,
    };

    let mousePos = {
      x: localPos.x,
      y: localPos.y,
    };

    mousePos = Affine.rotateAround(mousePos, -rotate, position);
    mousePos = {
      x: mousePos.x - position.x,
      y: mousePos.y - position.y,
    };

    let scale = {
      x: mousePos.x / corner.x,
      y: mousePos.y / corner.y,
    };

    let ratio = referScale.x / referScale.y;
    if (isHorizontalScale) {
      scale.y = scale.x / ratio;
    } else if (isVerticalScale) {
      scale.x = scale.y * ratio;
    } else {
      if (scale.x > scale.y * ratio) {
        scale = {
          x: scale.x,
          y: scale.x / ratio,
        };
      } else {
        scale = {
          x: scale.y * ratio,
          y: scale.y,
        };
      }
    }

    return {
      scale: scale,
    };
  }
}
